"""For help on EMAN, look up libpyEM instead of EMAN"""
from libpyEM import *
import os, os.path
import tempfile
import math
import time

EMANVERSION="EMAN2 v1.9"

EMData.MRC = Image.mrc
EMData.SPIDER = Image.spider
EMData.IMAGIC = Image.img
EMData.PGM = Image.pgm
EMData.LST = Image.lst
EMData.PIF = Image.pif
EMData.PNG = Image.png
EMData.HDF = Image.hdf5
EMData.DM3 = Image.dm3
EMData.TIF = Image.tif
EMData.VTK = Image.vtk
EMData.SAL = Image.sal
EMData.ICOS = Image.map
EMData.IMAGIC3D = Image.img3d
EMData.EMIM = Image.emim
EMData.SINGLE_SPIDER = Image.single_spider
EMData.GATAN = Image.dm2
EMData.AMIRA = Image.amira
EMData.XPLOR = Image.xplor
EMData.ANY = Image.any

Euler.EMAN = EulerType.eman
Euler.IMAGIC = EulerType.imagic
Euler.SPIN = EulerType.spin
Euler.QUAT = EulerType.quat
Euler.MATRIX = EulerType.matrix
Euler.SGIROT = EulerType.sgirot
Euler.SPIDER = EulerType.spider
Euler.MRC = EulerType.mrc

def displayphase(img):
	"""This will display complex phases of a single image with the phase-origin in the middle"""
	fsp=os.path.basename(tempfile.mktemp())+".mrc"
	if (img.isComplex()) : f=img
	else: f=img.doFFT()
	f.toCorner()
	f.ri2ap()
	for i in range(0,f.xSize()*f.ySize(),2):
		f.setValueAt(i,0,0,f.valueAt(i+1,0,0))
		f.setValueAt(i+1,0,0,0)
	f.ap2ri()

	f.writeImage(fsp)
	os.system("v2 "+fsp)
	os.unlink(fsp)
EMData.displayphase=displayphase

def pseudocomplex(img):
	"""This will create a volume data set from a complex volume where the left half of the image
	represents the phases and the right half represents amplitudes. Note that it is rescaled to
	be cubic in a lossy way."""
	if (type(img)==type("a")) :
		fsp=img
		img=EMData()
		img.readImage(fsp)
		img=img.doFFT()
		img.toCorner()
	if (not img.isComplex()) : return None
	img.ap2ri()
	sig=img.Sigma()
	img.ri2ap()
	ret=EMData()
	ret.setSize(img.xSize()-2,img.ySize(),img.zSize())
	for z in range(ret.zSize()):
		for y in range(ret.ySize()):
			for x in range(ret.xSize()/2):
				ret.setValueAt(ret.xSize()/2-x-1,y,z,img.valueAt(x*2+1,y,z)+math.pi)
				ret.setValueAt(x+ret.xSize()/2,y,z,img.valueAt(x*2,y,z)*math.pi/sig)
	ret.update()
	return ret

def ampcomplex(img):
	"""This will create a volume data set from a complex volume where both halves of the image
	represents amplitudes. Note that it is rescaled to
	be cubic in a lossy way."""
	if (type(img)==type("a")) :
		fsp=img
		img=EMData()
		img.readImage(fsp)
		img=img.doFFT()
		img.toCorner()
	if (not img.isComplex()) : return None
	img.ap2ri()
	sig=img.Sigma()
	img.ri2ap()
	ret=EMData()
	ret.setSize(img.xSize()-2,img.ySize(),img.zSize())
	for z in range(ret.zSize()):
		for y in range(ret.ySize()):
			for x in range(ret.xSize()/2):
				ret.setValueAt(ret.xSize()/2-x-1,y,z,img.valueAt(x*2,y,z))
				ret.setValueAt(x+ret.xSize()/2,y,z,img.valueAt(x*2,y,z))
	ret.update()
	return ret

def display(lst) :
	"""This is used to display a list of EMData objects"""
	fspbase = os.path.basename(tempfile.mktemp())
	fspimg = fspbase + ".img"
	fsphed = fspbase + ".hed"
	
	for i in lst:
		i.writeImage(fspimg, -1, Image.img)

	os.system("v2 " + fspimg)
	os.unlink(fspimg)
	os.unlink(fsphed)

def triplot(tuplelist):
	"""this will display a list of 2 or 3 tuples in a 'triview'"""
	out=os.popen("triplot -&","w")
	for i in tuplelist:
		if (len(i)==2) : out.write("%f\t%f\n"%(float(i[0]),float(i[1])))
		elif (len(i)>2) : out.write("%f\t%f\t%f\n"%(float(i[0]),float(i[1]),float(i[2])))
	out.close()

def readCTFParm(filename = "ctfparm.txt", return_type="list"):
	"""read in the CTF parameters and return in either a dictionary or a list"""
	ctfparm_dict = { }
	ctfparm_list = [ ]
	for l in open(filename):
		mid, ctfparm = l.strip().split();
		df, b, amp, ac, n1, n2, n3, n4, V, Cs, apix, sffile = ctfparm.split(",")
		ctfparm_dict[mid] = {"defocus":float(df), "B":float(b), "amplitude":amp,  "ac":ac, 
							"n1":float(n1), "n2":float(n2), "n3":float(n3), "n4":float(n4), 
							"V":float(V), "Cs":float(Cs), "apix":float(apix),  "sffile":sffile }
		ctfparm_list.append((mid, float(df), float(b), float(amp), float(ac), float(n1), float(n2), 
							float(n3), float(n4), float(V), float(Cs), float(apix), sffile))
		
	if return_type == "dict": return ctfparm_dict
	else: return ctfparm_list

def LOGbegin(ARGV):
	out=open(".emanlog","a")
	b=ARGV[0].split('/')
	ARGV[0]=b[len(b)-1]
	out.write("%d\t%d\t1\t%d\t%s\n"%(os.getpid(),time.time(),os.getppid()," ".join(ARGV)))
	out.close()

def LOGend():
	out=open(".emanlog","a")
	out.write("%d\t%d\t2\t-\t\n"%(os.getpid(),time.time()))
	out.close()
